//
//  IiideaInfoViewController.swift
//  sample
//
//  Created by CITIZEN Watch Co.,Ltd on 2019/10/29.
//  Copyright © 2019 CITIZEN Watch Co.,Ltd. All rights reserved.
//

import Foundation
import UIKit
import RiiiverSDK

class IiideaInfoViewController: UIViewController, UITextFieldDelegate {
    @IBOutlet weak var iiideaIcon: UIImageView!
    @IBOutlet weak var iiideaTitle: UILabel!
    @IBOutlet weak var activate: UISwitch!
    @IBOutlet weak var triggerInput: UITextField!
    @IBOutlet weak var removeIiidea: UIButton!
    
    @IBOutlet weak var triggerPieceCoreSample: UIButton!
    @IBOutlet weak var triggerPieceCoreSample2: UIButton!
    
    var iiidea:ERApplet?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.triggerInput.delegate = self
        iiideaIcon!.setImage(fromUrl:iiidea!.applet.iconUrl)
        iiideaTitle!.text = iiidea!.applet.getTitle()
        activate.isOn = iiidea!.activated
        
        if iiidea!.triggerBlockMetadata?.executor  == "TriggerPieceCoreSample" {
            // PieceのexecuterにTriggerPieceCoreSampleが指定されている場合
            // In case "executer" is set to "TriggerPieceCoreSample"
            triggerPieceCoreSample.isEnabled = iiidea!.activated        // disable UI, not for deactivating iiideat itself!
            triggerPieceCoreSample2.isEnabled = false                   // Activated UI, not for iiideat itself!
        }
        else if iiidea!.triggerBlockMetadata?.executor  == "TriggerPieceCoreSample2" {
            // PieceのexecuterにTriggerPieceCoreSample2が指定されている場合
            // In case "executer" is set to "TriggerPieceCoreSample2"
            triggerPieceCoreSample.isEnabled = false                    // disable UI, not for deactivating iiideat itself!
            triggerPieceCoreSample2.isEnabled = iiidea!.activated       // Activated UI, not for iiideat itself!
        }
        
        let appletIds:[String] = AppManager.shared.getDownloadAppletIds()
        appletIds.forEach { id  in
            if iiidea!.getId() == id {
                removeIiidea.isEnabled = false
            }
        }
    }
    
    @IBAction func activateChanged(_ sender: UISwitch) {
        guard let iiidea = iiidea else {
            print("activateChanged failed")
            return
        }
        
        if iiidea.triggerBlockUserPref == nil {
            if let defaultPref = iiidea.getTriggerBlockDefaultPref() {
                iiidea.setTriggerBlockUserPref(triggerUserPref: defaultPref)
            } else {
                iiidea.setTriggerBlockUserPref(triggerUserPref: [:])
            }
        }
        
        if iiidea.serviceBlockUserPref == nil {
            if let defaultPref = iiidea.getServiceBlockDefaultPref() {
                iiidea.setServiceBlockUserPref(serviceUserPref: defaultPref)
            } else {
                iiidea.setServiceBlockUserPref(serviceUserPref: [:])
            }
        }
        
        if iiidea.actionBlockUserPref == nil {
            if let defaultPref = iiidea.getActionBlockDefaultPref() {
                iiidea.setActionBlockUserPref(actionUserPref: defaultPref)
            } else {
                iiidea.setActionBlockUserPref(actionUserPref: [:])
            }
        }
        
        if(sender.isOn) {
            RiiiverSDKManager.shared.activate(iiidea: iiidea, completion: { (isSuccess, error) in
                if (isSuccess == false) {
                    print("activate failed")
                    let riiiverError:ERSDKError = error as! ERSDKError
                    if (riiiverError == ERSDKError.personalInfoError) {
                        // PersonalInfo画面へ  -  Segue to the Personal Info pane.
                        self.performSegue(withIdentifier: "toPersonalInfo", sender: self)
                        self.activate.setOn(false, animated:true)
                        return
                    }
                    self.activateFailedAlert(messeage: riiiverError.rawValue, isOn: false)
                    return
                }
                if iiidea.triggerBlockMetadata?.executor  == "TriggerPieceCoreSample" {
                    // PieceのexecuterにTriggerPieceCoreSampleが指定されている場合
                    self.triggerPieceCoreSample.isEnabled = iiidea.activated
                    self.triggerPieceCoreSample2.isEnabled = false
                }
                else if iiidea.triggerBlockMetadata?.executor  == "TriggerPieceCoreSample2" {
                    // PieceのexecuterにTriggerPieceCoreSample2が指定されている場合
                    self.triggerPieceCoreSample.isEnabled = false
                    self.triggerPieceCoreSample2.isEnabled = iiidea.activated
                }
            })
        } else {
            RiiiverSDKManager.shared.deactivate(iiidea: iiidea, completion: { (isSuccess, error) in
                if (isSuccess == false) {
                    print("deactivate failed")
                    let riiiverError:ERSDKError = error as! ERSDKError
                    self.activateFailedAlert(messeage: riiiverError.rawValue, isOn: true)
                    return
                }
                self.triggerPieceCoreSample.isEnabled = false
                self.triggerPieceCoreSample2.isEnabled = false
            })
        }
    }
    
    func activateFailedAlert(messeage:String, isOn:Bool) {
        let alert: UIAlertController = UIAlertController(title: "Activate Failed", message: messeage, preferredStyle:  UIAlertController.Style.alert)
        let defaultAction: UIAlertAction = UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: {
            (action: UIAlertAction!) -> Void in
            self.activate.setOn(isOn, animated:true)
        })
        alert.addAction(defaultAction)
        present(alert, animated: true, completion: nil)
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        triggerInput.resignFirstResponder()
        return true
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        if (self.triggerInput.isFirstResponder) {
            self.triggerInput.resignFirstResponder()
        }
    }
    
    @IBAction func triggerPieceCoreSample(_ sender: Any) {
        guard let iiidea = iiidea else {
            print("trigger failed. reason: No iiidea")
            return
        }
        
        print("preference:\(iiidea.triggerBlockUserPref ?? [:])")
        
        var userInfo: [String:Any] = [:]
        userInfo[TriggerNotification.Key.Identifier] = iiidea.getId()
        userInfo[TriggerNotification.Key.Output] = triggerInput.text
        
        // Notify the execution of the iiidea - iiidea実行の通知
        NotificationCenter.default.post(name: TriggerNotification.Name.sampleTrigger, object: nil, userInfo: userInfo)
    }
    
    @IBAction func triggerPieceCoreSample2(_ sender: Any) {
        guard let iiidea = iiidea else {
            print("trigger failed. reason: No iiidea")
            return
        }
        
        print("preference:\(iiidea.triggerBlockUserPref ?? [:])")
        
        var userInfo: [String:Any] = [:]
        userInfo[TriggerNotification.Key.Identifier] = iiidea.getId()
        userInfo[TriggerNotification.Key.Output] = triggerInput.text
        
        // Notify the execution of the iiidea - iiidea実行の通知
        NotificationCenter.default.post(name: TriggerNotification.Name.sampleTrigger2, object: nil, userInfo: userInfo)
    }
        
    @IBAction func back(_ sender: Any) {
        if let sampleViewController = self.presentingViewController as? SampleViewController {
            sampleViewController.isReload = false
        }
        self.dismiss(animated: true, completion: nil)
    }
    
    @IBAction func goToSetting(_ sender: Any) {
        // iiide設定画面へ  -  Segue to the iiidea setting pane.
        self.performSegue(withIdentifier: "toSetting", sender: self)
    }
    
    @IBAction func removeiiidea(_ sender: Any) {
        guard let iiidea = iiidea else {
            print("removeiiidea. reason: No iiidea")
            return
        }
        //UIAlertControllerのスタイルがalert
        let confirmAction: UIAlertAction = UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler:{
            (action: UIAlertAction!) -> Void in
            RiiiverSDKManager.shared.removeMyIiidea(iiidea:iiidea, completion: { (isSuccess, error) in
                if (isSuccess == false) {
                    print("removeiiidea failed")
                    let okAction: UIAlertAction = UIAlertAction(title: "OK", style: UIAlertAction.Style.cancel, handler: nil)
                    self.showAlert(title: "Delete Failed", message:  "", actions: [okAction])
                    return
                }
                if let sampleViewController = self.presentingViewController as? SampleViewController {
                    sampleViewController.isReload = true
                }
                self.dismiss(animated: true, completion: nil)
            })
        })
        let cancelAction: UIAlertAction = UIAlertAction(title: "CANCEL", style: UIAlertAction.Style.cancel, handler: nil)
        showAlert(title: "Delete the iiidea?", message:  "You can download the iiidea you deleted from the store again.", actions: [confirmAction, cancelAction])
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let IiideaSettingViewController = segue.destination as? IiideaSettingViewController {
            IiideaSettingViewController.iiidea = iiidea
        }
        if let PersonalInfoViewController = segue.destination as? PersonalInfoViewController {
            PersonalInfoViewController.iiidea = iiidea
        }
    }
    
    private func showAlert(title:String, message:String, actions:[UIAlertAction]) {
        let alert: UIAlertController = UIAlertController(title: title, message:  message, preferredStyle:  UIAlertController.Style.alert)
        for action in actions {
            alert.addAction(action)
        }
        present(alert, animated: true, completion: nil)
    }
}
