//
//  IiideaSettingViewController.swift
//  sample
//
//  Created by CITIZEN Watch Co.,Ltd on 2019/11/08.
//  Copyright © 2019 CITIZEN Watch Co.,Ltd. All rights reserved.
//

import Foundation
import UIKit
import RiiiverSDK
import WebKit
import CoreLocation
import MediaPlayer
import Speech
import EventKit
import AuthenticationServices


class IiideaSettingViewController: UIViewController, ERPreferencesViewDelegate, ERPreferencesViewPermissionDelegate, CLLocationManagerDelegate {
    /// iiideaPreferences用WebView
    @IBOutlet weak var webPreference: ERPreferencesView!
    
    let locationManager = CLLocationManager()
    let calenderManager = EKEventStore()
    var session: ASWebAuthenticationSession?
    
    private var urlTmpBaseFolder: URL!
    var iiidea:ERApplet?
    
    override func viewDidLoad() {
        // Preference用のWebコンテンツをアプリ領域にコピー
        // Copy the contents for "Preference" Web view.
        copyBaseFile()
        
        // load Preference view - Preferences画面起動
        self.loadPreferences(iiidea: iiidea!, urlBaseFolder: urlTmpBaseFolder)
        webPreference.preferencesDelegate = self
        webPreference.permissionDelegate = self
        
        NotificationCenter.default.addObserver(self, selector: #selector(IiideaSettingViewController.viewWillEnterForeground(_:)),name: UIApplication.willEnterForegroundNotification,object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(IiideaSettingViewController.viewWillEnterBackground(_:)), name: UIApplication.didEnterBackgroundNotification,object: nil)
    }
    
    @objc func viewWillEnterForeground(_ notification: Notification?) {
        print("viewWillEnterForeground")
        // バックグラウンドに行っている間にアプリのパーミッション設定が変わっている可能性があるので
        // フォアグラウンドになったらPreferencesViewをリロードする
        // In case app view goes to background when a permission setting view is showing:
        // The permission view has to be reloaded when it goes foreground.
        self.reloadPreferences()
    }
    
    @objc func viewWillEnterBackground(_ notification: Notification?) {
        print("viewWillEnterBackground")
    }
    
    private func loadPreferences(iiidea: ERApplet, urlBaseFolder: URL) {
        self.webPreference.loadPreferences(applet: iiidea, urlBaseFolder: urlBaseFolder)
    }

    private func reloadPreferences() {
        DispatchQueue.main.async {
            self.webPreference.reloadPreferences()
        }
    }

    // Preference画面から閉じる要求があった場合に呼ばれる
    func closePreferences(applet: ERApplet) {
        // PreferenceのWebContentに閉じるボタンを表示している場合に呼ばれる。
        // デフォルトでは呼ばれることはない。
    }

    // TriggerPieceのPreferenceが更新された場合に呼ばれる
    // This function will be called when TrigerPiece's Preference is updated.
    func updateTriggerPreferences(applet: ERApplet) {
        print("updateTriggerPreferences")
    }
    
    // ServicePieceのPreferenceが更新された場合に呼ばれる
    // This function will be called when ServicePiece's Preference is updated.
    func updateServicePreferences(applet: ERApplet) {
        print("updateServicePreferences")
    }
    
    // ActionPieceのPreferenceが更新された場合に呼ばれる
    // This function will be called when ActionPiece's Preference is updated.
    func updateActionPreferences(applet: ERApplet) {
        print("updateActionPreferences")
    }
    
    // This function will be called when we need permission to access location/microphone/calendar/mediaLibraty.  - Permission表示が必要な場合に呼ばれる
    func requestPermissionScreen(permissionType: String) {
        print("requestPermissionScreen")
        
        // PermissionTypeに応じてPermission要求を画面に表示する
        // The suitable view will be displayed depending on the PermissionType.
        switch permissionType {
        case ERPermissionType.auth.rawValue: break
        case ERPermissionType.location.rawValue:
            self.showCLLocationManagerPermission()
        case ERPermissionType.microphone.rawValue:
            self.showAVCaptureDeviceAudioPermission(
                completion: {(status) -> Void in
                    self.reloadPreferences()
                })
        case ERPermissionType.calendar.rawValue:
            self.showEKEventStorePermission(
                completion: {() -> Void in
                    self.reloadPreferences()
                })
        case ERPermissionType.mediaLibrary.rawValue:
            self.showMPMediaLibraryPermission(
                completion: {(status) -> Void in
                    self.reloadPreferences()
                })
        case ERPermissionType.notification.rawValue:
            self.showUNUserNotificationCenterPermission(
                completion: {(status) -> Void in
                    self.reloadPreferences()
                })
        default: break
        }
    }
    
    // Auth認証が必要な場合に呼ばれる
    // Called when O-Auth is needed.
    func requestAuthScreen(blockId: String) {
        print("requestAuthScreen")
        let cancelHandle = UIAlertAction(title: "CANCEL", style: .default)
        let okHandle =  UIAlertAction(title: "OK", style: .default) { (action: UIAlertAction!) in
            RiiiverSDKManager.shared.showAuthenticationSession(blockId: blockId) { isSuccess, error  in
                if let e = error {
                    print("[requestAuthScreen] requestAuthScreen is failed: \(e)")
                }
                if(isSuccess == false) {
                    let alert = UIAlertController(title: "", message: "Display failed.", preferredStyle: .alert)
                    alert.addAction(UIAlertAction(title: "OK", style: .default))
                    self.present(alert, animated: true, completion: nil)
                }
            }
        }
        // OAuth連携確認用のダイアログを表示
        let alert = UIAlertController(title: "", message: "Agree cooperation with service?", preferredStyle: .alert)
        alert.addAction(cancelHandle)
        alert.addAction(okHandle)
        self.present(alert, animated: true, completion: nil)
    }
    
    // Auth認証の解除が必要な場合に呼ばれる
    func requestDeleteAuthScreen(blockId: String) {
        print("requestDeleteAuthScreen")
        let cancelHandle = UIAlertAction(title: "CANCEL", style: .default)
        let okHandle =  UIAlertAction(title: "OK", style: .default) { (action: UIAlertAction!) in
            RiiiverSDKManager.shared.stopAuthenticationSession(blockId: blockId) { isSuccess, error  in
                if let e = error {
                    print("[AppletBuilder] deleteClientAuth is failed: \(e)")
                    let alert = UIAlertController(title: "", message: "Delete failed.", preferredStyle: .alert)
                    alert.addAction(UIAlertAction(title: "OK", style: .default))
                    self.present(alert, animated: true, completion: nil)
                }
                else {
                    self.reloadPreferences()
                }
            }
        }
        // OAuth連携解除確認用のダイアログを表示
        let alert = UIAlertController(title: "", message: "Stop cooperation with service?", preferredStyle: .alert)
        alert.addAction(cancelHandle)
        alert.addAction(okHandle)
        self.present(alert, animated: true, completion: nil)
    }
    
    @IBAction func back(_ sender: Any) {
        NotificationCenter.default.removeObserver(self, name: UIApplication.willEnterForegroundNotification, object: nil)
        NotificationCenter.default.removeObserver(self, name: UIApplication.didEnterBackgroundNotification, object: nil)
        self.dismiss(animated: true, completion: nil)
    }
    
    private func copyBaseFile() {
        let baseFolderName: String = "Preferences"
        let directoryName = "ERSDK_preferences"
        
        let fromPath_tmp: NSString = (Bundle.main.bundlePath) as NSString
        let fromPath: String = fromPath_tmp.appendingPathComponent("WebContent/" + baseFolderName)
        do {
            let manager:FileManager = FileManager.default
            let urlBaseFolder:URL = URL(fileURLWithPath: fromPath)
            let urlTmpBaseFolder:URL = URL(fileURLWithPath: NSTemporaryDirectory() + directoryName)
            
            // バンドルしているHtmlをコピー
            // Copy bandle HTML files.
            if manager.fileExists(atPath: urlTmpBaseFolder.path) == true {
                try manager.removeItem(at: urlTmpBaseFolder)
            }
            try manager.copyItem(at: urlBaseFolder, to: urlTmpBaseFolder)
            
            self.urlTmpBaseFolder = urlTmpBaseFolder
        }
        catch {
            print("copyBaseFile: error \(error)")
        }
    }
    
    /// 位置情報のパーミッション許可画面を表示する。
    /// Display a permission view for current location of the users.
    ///
    func showCLLocationManagerPermission() {
        let status = CLLocationManager.authorizationStatus()
        switch status {
        case .notDetermined:
            // Not determined when the load is the first time. - 初回起動時はnotDetermined
            locationManager.requestWhenInUseAuthorization()
            locationManager.delegate = self
            break
        default:
            // In the case, it was once denied. - 一度拒否されている場合
            if let url = URL(string: UIApplication.openSettingsURLString) {
                DispatchQueue.main.async {
                    // 設定画面をオープン
                    UIApplication.shared.open(url, options: [:], completionHandler: nil)
                }
            }
            break
        }
    }
    
    
    /// メディアライブラリのパーミッション許可画面を表示する。
    /// Display a permission view to access the media libraries.
    ///
    /// - Parameters:
    ///   - completion: 初回起動時によるパーミッション許可通知
    func showMPMediaLibraryPermission(completion: ((MPMediaLibraryAuthorizationStatus?) -> Void)? = nil) {
        let status = MPMediaLibrary.authorizationStatus()
        switch status {
        case .notDetermined:
            // 一度も許可/拒否していない場合は許可リクエストをコール
            MPMediaLibrary.requestAuthorization { status in
                if status == .authorized {
                    if let completion = completion {
                        completion(status)
                    }
                }
            }
            break
        default:
            // In the case, it was once denied - 一度拒否されている場合
            // Open settings view - 設定画面を開く
            openSettings()
            break
        }
    }
    
    /// マイクのパーミッション許可画面を表示する。
    /// Display a permission view to access a microphone.
    ///
    /// - Parameters:
    ///   - completion: The notification of the first permission request. - 初回起動時によるパーミッション許可通知
    func showAVCaptureDeviceAudioPermission(completion: ((Bool?) -> Void)? = nil) {
        let status = AVCaptureDevice.authorizationStatus(for: .audio)
        switch status {
        case .notDetermined:
            // Not determined when the load is the first time. - 初回起動時はnotDetermined
            AVCaptureDevice.requestAccess(for: .audio) { authentication in
                if (authentication == true) {
                    if let completion = completion {
                        completion(authentication)
                    }
                }
            }
            break
        default:
            // In the case, it was once denied - 一度拒否されている場合
            // Open settings view - 設定画面を開く
            openSettings()
            break
        }
    }
    
    /// カレンダーのパーミッション許可画面を表示する。
    /// Display a permission view to access a calender.
    ///
    /// - Parameters:
    ///   - completion: The notification of the first permission request. - 初回起動時によるパーミッション許可通知
    func showEKEventStorePermission(completion: (() -> Void)? = nil) {
        let status = EKEventStore.authorizationStatus(for: .event)
        switch status {
        case .notDetermined:
            // Not determined when the load is the first time. - 初回起動時はnotDetermined
            EKEventStore().requestAccess(to: .event) { granted, error in
                if (granted == true && error == nil) {
                    if let completion = completion {
                        completion()
                    }
                }
            }
        default:
            // In the case, it was once denied - 一度拒否されている場合
            // Open settings view - 設定画面を開く
            openSettings()
            break
        }
    }

    /// 通知許可画面を表示する。
    ///
    /// - Parameters:
    ///   - completion: The notification of the first permission request. - 初回起動時によるパーミッション許可通知
    func showUNUserNotificationCenterPermission(completion: ((Bool?) -> Void)? = nil) {
        UNUserNotificationCenter.current().getNotificationSettings(){setting in
            switch setting.authorizationStatus {
            case .notDetermined:
                let authOptions: UNAuthorizationOptions = [.alert, .sound]
                UNUserNotificationCenter.current().requestAuthorization(options: authOptions, completionHandler: { granted, error in
                    if (granted == true && error == nil) {
                        if let completion = completion {
                            completion(granted)
                        }
                    }
                })
            default:
                // In the case, it was once denied. - 一度拒否されている場合
                if let url = URL(string: UIApplication.openSettingsURLString) {
                    DispatchQueue.main.async {
                        // 設定画面をオープン
                        UIApplication.shared.open(url, options: [:], completionHandler: nil)
                    }
                }
                break
            }
        }
    }

    /// iOSの設定画面を開く
    /// Open the Settings view of the iOS.
    func openSettings() {
        if let url = URL(string: UIApplication.openSettingsURLString) {
            DispatchQueue.main.async {
                // 設定画面をオープン
                UIApplication.shared.open(url, options: [:], completionHandler: nil)
            }
        }
    }
    
    func locationManager(_ manager: CLLocationManager, didChangeAuthorization status: CLAuthorizationStatus) {
        self.reloadPreferences()
    }
}
