//
//  SampleViewController.swift
//  sample
//
//  Created by CITIZEN Watch Co.,Ltd on 2019/10/28.
//  Copyright © 2019 CITIZEN Watch Co.,Ltd. All rights reserved.
//

import Foundation
import UIKit
import RiiiverSDK

class SampleViewController: UIViewController, UITableViewDelegate, UITableViewDataSource {
    @IBOutlet weak var tableView: UITableView!
    @IBOutlet weak var newLabel: UILabel!

    var iiideas: [ERApplet]?

    var indicator = UIActivityIndicatorView()
    
    var iiideaIndex:Int = 0
    
    var isReload:Bool = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        indicator.center = view.center
        indicator.style = .whiteLarge
        indicator.color = .darkGray
        view.addSubview(indicator)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        if (isReload == true) {
            indicator.startAnimating()
            // RiiiverSDKからiiidea一覧を取得   -   Get an iiidea list from RiiiverSDK
            RiiiverSDKManager.shared.getMyApplets(completion: { (isSuccess, iiideas, error) in
                self.indicator.stopAnimating()
                if (isSuccess == false) {
                    print("getMyApplets failed")
                    // 失敗した場合であってもiiide実行サンプル画面へ(何もできなくなるので)
                    // Even if we got errors, keep going to the iiidea executer pane.
                }
                self.iiideas = iiideas
                self.tableView.reloadData()
            })
        }
        RiiiverSDKManager.shared.isUnreadNews() { isUnread in
            if (isUnread == false) {
                // 未読無しなので非表示
                self.newLabel!.isHidden = true
            }
            else {
                // 未読有りなので表示
                self.newLabel!.isHidden = false
            }
        }
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        guard let iiideas = self.iiideas else {
            return 0
        }
        return iiideas.count
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 90
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell: SampleViewCell = tableView.dequeueReusableCell(withIdentifier: "cell", for: indexPath) as! SampleViewCell
        if let iiideas = self.iiideas {
            cell.icon!.setImage(fromUrl:iiideas[indexPath.row].applet.iconUrl)
            cell.title!.text = iiideas[indexPath.row].applet.getTitle()
            cell.title!.textColor =  .gray
            let appletIds:[String] = AppManager.shared.getDownloadAppletIds()
            appletIds.forEach { id  in
                if iiideas[indexPath.row].getId() == id {
                    cell.title!.textColor = .red
                }
            }
        }
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        print("tableView index \(indexPath.row)")
        iiideaIndex = indexPath.row
        // iiide情報画面へ  -  Segue to the iiidea infomation pane.
        self.performSegue(withIdentifier: "toIiideaInfo", sender: self)
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let iiideaInfoViewController = segue.destination as? IiideaInfoViewController {
            if let iiideas = self.iiideas {
                iiideaInfoViewController.iiidea = iiideas[iiideaIndex]
            }
        }
    }

    @IBAction func openProfile(_ sender: Any) {
        self.performSegue(withIdentifier: "toProfile", sender: self)
    }

    @IBAction func openNews(_ sender: Any) {
        self.performSegue(withIdentifier: "toNews", sender: self)
    }

    @IBAction func openStore(_ sender: Any) {
        self.performSegue(withIdentifier: "toStore", sender: self)
    }
}
