class ERSDK {
  constructor() {
    this.UUID = "0a565343-1923-9725-78e2-0767d2786533_";
    this.ICON_TYPE = { IIIDEA: "iiidea", TRIGGER: "trigger", SERVICE: "service", ACTION: "action" };
    this.PIECE_TYPE = { TRIGGER: "trigger", SERVICE: "service", ACTION: "action" };
    this.VALUE_TYPE = { STRING: "string", INTEGER: "integer", NUMBER: "number", BOOL: "boolean", ARRAY: "array" };
    this.PERMISSION_TYPE = { LOCATION: "location", MICROPHONE: "microphone", CALENDER: "calender", MEDIALIBRARY: "mediaLibrary",  NOTIFICATION: "notification",  NOTIFICATIONACCESS: "notificationAccess", AUTH: "auth" };

    this.CALENDAR_STATE = { NEXT: 1, PREV: 2 };
    this.MONTH = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
    this.WEEKS = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];

    this.isInitialized = false;
    this.isMapInitialized = false;
    this.isLoadedHTML = false;

    this.initCallbackIds = 0;
    this.initCallbackList = new Object();
    this.loadedHTMLCallbackIds = 0;
    this.loadedHTMLCallbackList = new Object();
    this.loadingCompletedHTML = new Object();
    this.googleMapCallbackIds = 0;
    this.googleMapCallbackList = new Object();
    this.requestCommCallbackIds = 0;
    this.requestCommCallbackList = new Object();
    this.showAuthPageCallbackIds = 0;
    this.showAuthPageCallbackList = new Object();
    this.setAuthInfoCallbackIds = 0;
    this.setAuthInfoCallbackList = new Object();
    this.getIconImageCallbackIds = 0;
    this.getIconImageCallbackList = new Object();
    this.deleteIiideaCallbackIds = 0;
    this.deleteIiideaCallbackList = new Object();
    this.changeActivateStatusCallbackIds = 0;
    this.changeActivateStatusCallbackList = new Object();
    this.calendarUpdateCallbackList = new Object();

    this._nativeCall({ api: "nativeInit" });

    window.onerror = function (msg) {
      // Error監視開始
      erSDK.log("Error", msg);
    };

    window.addEventListener('load', function () {
      erSDK.log("addEventListener load", `in`);
      erSDK.isLoadedHTML = true;
      // HTML読み込み完了
      for (let key in erSDK.loadedHTMLCallbackList) {
        erSDK.loadedHTMLCallbackList[key]();
        delete erSDK.loadedHTMLCallbackList[key];
      }
      // HTML読み込み完了時に行う処理が全て完了
      erSDK.loadingCompletedHTML();
    });
  }

  /**
   * erSDKの初期化完了通知用コールバックを登録する
   * @param {initCallback} callback コールバック関数
   * @return {number} コールバックID(既に完了済みの場合-1)
   */
  addInitCallback(callback) {
    this.log("addInitCallback", `in`);
    if (this.isInitialized === true) {
      callback();
      return -1;
    }
    let id = this.initCallbackIds;
    this.initCallbackList[id] = callback;
    this.initCallbackIds++;
    return id;
  }

  /**
   * 画面読み込み完了通知用コールバックを登録する
   * @param {initCallback} callback コールバック関数
   * @return {number} コールバックID(既に完了済みの場合-1)
   */
   addLoadedHTMLCallback(callback) {
    this.log("addLoadedHTMLCallback", `in`);
    if (this.isLoadedHTML === true) {
      callback();
      return -1;
    }
    let id = this.loadedHTMLCallbackIds;
    this.loadedHTMLCallbackList[id] = callback;
    this.loadedHTMLCallbackIds++;
    return id;
  }

  /**
   * Preferences画面を再描画する
   */
   reload() {
    this.log("reload", `in`);
    this._nativeCall({
      api: "reload",
    });
  }

  /**
   * PieceJsonのCustomOrder情報を取得する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   */
   getCustomOrder(pieceType) {
    this.log("getCustomOrder", `in`);
    switch (pieceType) {
      case this.PIECE_TYPE.TRIGGER:
        return this.triggerCustomOrder;
      case this.PIECE_TYPE.SERVICE:
        return this.serviceCustomOrder;
      case this.PIECE_TYPE.ACTION:
        return this.actionCustomOrder;
      default:
        this.log("setPreferences", `Piece Name Error`);
        return;
    }
  }

  /**
   * erSDKの初期化完了通知用コールバックを登録解除する
   * @param {number} id コールバックID
   */
  removeInitCallback(id) {
    this.log("removeInitCallback", `in: id=${id}`);
    delete this.initCallbackList[id];
  }

  /**
   * googleMapの初期化完了通知用コールバックを登録する
   * 登録したコールバックは必ずerSDK初期化完了後に通知される
   * @param {googleMapCallback} callback コールバック関数
   * @return {number} コールバックID
   */
  addGoogleMapCallback(callback) {
    this.log("addGoogleMapCallback", `in`);
    if (this.isMapInitialized === true) {
      callback();
      return -1;
    }
    let id = this.googleMapCallbackIds;
    this.googleMapCallbackList[id] = callback;
    this.googleMapCallbackIds++;
    return id;
  }

  /**
   * googleMapの初期化完了通知用コールバックを登録解除する
   * @param {number} id コールバックID
   */
  removeGoogleMapCallback(id) {
    this.log("removeGoogleMapCallback", `in: id=${id}`);
    delete this.googleMapCallbackList[id];
  }

  /**
   * RiiiverのCredentialを取得する
   * @return {string} credential
   */
  getCredential() {
    this.log("getCredential", `in`);
    return this.credential;
  }

  /**
   * 言語コードを取得する
   * @return {string} 言語コード情報(Locale)
   */
  getLanguage() {
    this.log("getLanguage", `in`);
    return this.language;
  }

  /**
   * 国コードを取得する
   * @return {string} 国コード情報(Locale)
   */
  getCountryCode() {
    this.log("getCountryCode", `in`);
    return this.countryCode;
  }

  /**
   * iiideaデフォルト設定用のPreferenceか否かを示すフラグを取得する
   * @return {boolean} iiideaデフォルト設定用のPreferenceの場合はtrue
   */
  isIiideaDefaultSetting() {
    this.log("isIiideaDefaultSetting", `in`);
    return this.isAppletDefaultPref;
  }

  /**
   * カレンダーを表示する
   * @param {string} id 更新するカレンダーTagのid
   * @param {number} year 表示する年
   * @param {number} month 表示する月
   * @param {number[]} select 選択状態にする日付情報の配列 [0]=yyyy, [1]=mm, [2]=dd
   * @param {updateCalendarSelect} callback callback コールバック関数
   */
  showCalendar(id, year, month, select, callback) {
    this.log("showCalendar", `in: id=${id},  year=${year}, month=${month}, select=${select}`);
    if (callback) {
      this.calendarUpdateCallbackList[id] = callback;
    }

    // カレンダー構築
    const calendarHtml = this.createCalendar(year, month);
    const sec = document.createElement('section');
    sec.innerHTML = calendarHtml;

    // 既にカレンダー情報が設定されている可能性があるので必ず初期化してから設定
    document.querySelector(id).innerHTML = '';
    document.querySelector(id).appendChild(sec);

    let days = document.querySelector(id).getElementsByClassName('day');
    let daysValue = document.querySelector(id).getElementsByClassName('dayValue');
    for (let i = 0; i < days.length; i++) {
      // タップを監視
      daysValue[i].addEventListener('click', function (element) {
        let selectElement = document.querySelector(id).getElementsByClassName('day focus');
        if (selectElement.length != 0) {
          selectElement[0].classList.remove('focus');
        }
        // タップされたセルを選択状態に
        element.target.parentNode.classList.add('focus');
        let callback = erSDK.calendarUpdateCallbackList[id];
        let date = element.target.dataset;
        callback(parseInt(date.year), parseInt(date.month), parseInt(date.day));
      });

      // selectチェック
      if (!Array.isArray(select)) {
        this.log("showCalendar", `param 'select' none.`);
        return;
      }
      if (select.length < 2) {
        this.log("showCalendar", `param 'select' is not enough.[${select.length}]`);
        return;
      }
      for (let i in select) {
        if (isNaN(select[i])) {
          this.log("showCalendar", `param 'select' is not a number.[${select[i]}]`);
          return;
        }
      }
      // 指定された「選択状態にする日付」と一致する日付を選択状態に
      let dataset = daysValue[i].dataset;
      if (parseInt(select[0]) === parseInt(dataset.year) &&
        parseInt(select[1]) === parseInt(dataset.month) &&
        parseInt(select[2]) === parseInt(dataset.day)) {
        days[i].classList.add('focus');
      }
    }
  }

  /**
   * カレンダーを構築する
   * @param {number} year 表示する年
   * @param {number} month 表示する月
   */
  createCalendar(year, month) {
    this.log("createCalendar", `in: year=${year},  month=${month}`);
    const startDate = new Date(year, month - 1, 1);          // 月の最初の日を取得
    const endDate = new Date(year, month, 0);                // 月の最後の日を取得
    const endDayCount = endDate.getDate();                   // 月の末日
    const lastMonthEndDate = new Date(year, month - 1, 0);   // 前月の最後の日の情報
    const lastMonthendDayCount = lastMonthEndDate.getDate(); // 前月の末日
    const startDay = startDate.getDay();                     // 月の最初の日の曜日を取得
    const today = new Date()
    let dayCount = 1;
    let calendarHtml = '';

    calendarHtml += `<h1 style="display:none" class="calendar_month" data-year="${year}" data-month="${month}">${year}/${month}</h1>`;
    calendarHtml += '<table>';

    // 曜日の行(ヘッダ)を作成
    for (let i = 0; i < this.WEEKS.length; i++) {
      calendarHtml += '<th>' + this.WEEKS[i] + '</th>';
    }

    for (let w = 0; w < 6; w++) {
      calendarHtml += '<tr>';
      for (let d = 0; d < 7; d++) {
        if (w == 0 && d < startDay) {
          // 1行目で1日の曜日の前
          let num = lastMonthendDayCount - startDay + d + 1;
          calendarHtml += '<td class="is-disabled"><div class="disabledDay">' + num + '</div></td>';
        } else if (dayCount > endDayCount) {
          // 末尾の日数を超えた
          let num = dayCount - endDayCount;
          calendarHtml += '<td class="is-disabled"><div class="disabledDay">' + num + '</div></td>';
          dayCount++;
        } else {
          let todayCss = "";
          if (year === today.getFullYear() && month === today.getMonth() + 1 && dayCount === today.getDate()) {
            todayCss = "today"
          }
          calendarHtml += `<td class="calendar_day"><div class="day ${todayCss}"><div class="dayValue" data-year="${year}"data-month="${month}"data-day="${dayCount}">${dayCount}</div></div></td>`;
          dayCount++;
        }
      }
      calendarHtml += '</tr>';
    }
    calendarHtml += '</table>';
    return calendarHtml
  }

  /**
   * 先月、または翌月のカレンダーを表示する
   * @param {string} id 更新するカレンダーTagのid
   * @param {number} nextState 表示する月を CALENDAR_STATE で指定
   * @param {number[]} select 選択状態にする日付情報の配列 [0]=yyyy, [1]=mm, [2]=dd
   * @return {number[]} 現在表示している日付情報の配列 [0]=yyyy, [1]=mm
   */
  moveCalendar(id, nextState, select) {
    this.log("moveCalendar", `in: id=${id}, nextState=${nextState}, select=${select}`);
    let element = document.querySelector(id).getElementsByClassName('calendar_month');
    if (element.length == 0) {
      return;
    }
    let year = parseInt(element[0].dataset.year);
    let month = parseInt(element[0].dataset.month);
    switch (nextState) {
      case this.CALENDAR_STATE.PREV:
        month--;
        if (month < 1) {
          year--;
          month = 12;
        }
        break;
      case this.CALENDAR_STATE.NEXT:
        month++;
        if (month > 12) {
          year++;
          month = 1;
        }
        break;
    }
    this.showCalendar(id, year, month, select);
    return [year, month]
  }

  /**
   * タグidからPrefixを除去する
   * @param {string} id selectタグのid
   * @return {string} Prefixを除去したタグid
   */
  removePrefix(id) {
    this.log("removePrefix", `in: id=${id}`);
    let index = id.indexOf('_');
    return id.substr(index + 1);
  }

  /**
   * selectタグのvalueを保存する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @param {string} type valueのType
   * @param {string} id selectタグのid
   */
  setSelectValue(pieceType, type, id) {
    this.log("setSelectValue", `in: pieceType=${pieceType}, type=${type}, id=${id}`);
    let obj = document.getElementById(id);
    let index = obj.options.selectedIndex;
    erSDK.setPreferences(pieceType, this.removePrefix(id), type, obj.options[index].value);
  }

  /**
   * Preferences情報を保存する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @param {string} key 保存情報のフィールド名
   * @param {string} type dataのType
   * @param {JSON} data 保存対象のPreferences情報
   */
  setPreferences(pieceType, key, type, data) {
    this.log("setPreferences", `in: pieceType=${pieceType}, key=${key}, type=${type}, data=${data}`);
    let pref;
    let value;

    switch (type) {
      case this.VALUE_TYPE.STRING:
        value = String(data);
        break;
      case this.VALUE_TYPE.INTEGER:
        value = parseInt(data);
        break;
      case this.VALUE_TYPE.NUMBER:
        value = Number(data);
        break;
      case this.VALUE_TYPE.BOOL:
        value = Boolean(data);
        break;
      case this.VALUE_TYPE.ARRAY:
        value = data;
        break;
    }

    switch (pieceType) {
      case this.PIECE_TYPE.TRIGGER:
        this.triggerPref[key] = value;
        pref = this.triggerPref;
        break;
      case this.PIECE_TYPE.SERVICE:
        this.servicePref[key] = value;
        pref = this.servicePref;
        break;
      case this.PIECE_TYPE.ACTION:
        this.actionPref[key] = value;
        pref = this.actionPref;
        break;
      default:
        this.log("setPreferences", `Piece Name Error`);
        return;
    }

    this._nativeCall({
      api: "setPreferences",
      values: {
        "pieceType": pieceType,
        "pref": pref
      }
    });
  }

  /**
   * Preferences情報を取得する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @param {string} key 保存情報のフィールド名
   * @return {JSON} Preferences情報
   */
  getPreferences(pieceType, key) {
    this.log("getPreferences", `in: pieceType=${pieceType}, key=${key}`);
    let ret = undefined;
    let pref = undefined;
    switch (pieceType) {
      case this.PIECE_TYPE.TRIGGER:
        pref = this.triggerPref[key];
        break;
      case this.PIECE_TYPE.SERVICE:
        pref = this.servicePref[key];
        break;
      case this.PIECE_TYPE.ACTION:
        pref = this.actionPref[key];
        break;
      default:
        this.log("getPreferences", `Piece Name Error`);
        return ret;
    }
    return pref;
  }

  /**
   * Preferences情報を削除する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @param {string} key 削除情報のフィールド名
   */
  removePreferences(pieceType, key) {
    this.log("removePreferences", `in: pieceType=${pieceType}, key=${key}`);
    let pref;

    switch (pieceType) {
      case this.PIECE_TYPE.TRIGGER:
        delete this.triggerPref[key];
        pref = this.triggerPref;
        break;
      case this.PIECE_TYPE.SERVICE:
        delete this.servicePref[key];
        pref = this.servicePref;
        break;
      case this.PIECE_TYPE.ACTION:
        delete this.actionPref[key];
        pref = this.actionPref;
        break;
      default:
        this.log("removePreferences", `Piece Name Error`);
        return;
    }

    this._nativeCall({
      api: "setPreferences",
      values: {
        "pieceType": pieceType,
        "pref": pref
      }
    });
  }

  /**
   * TextUIの仕様に合わせて情報を保存or削除する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @param {string} id 保存情報フィールドのid
   * @param {string} type dataのType
   * @param {JSON} data 保存対象のPreferences情報
   * @param {JSON} defval 保存対象のデフォルト情報
   */
  updateTextPreferences(pieceType, id, type, data, defval) {
    this.log("updateTextPreferences", `in: pieceType=${pieceType}, id=${id}, data=${data}`);
    let key = this.removePrefix(id);
    let defExists = this.defaultExists(pieceType, key);
    let optional = this.isOptional(pieceType, key);
    if(optional === true) {
      this.setPreferences(pieceType, key, type, data);
    }
    else {
      if (data === ""){
        if (defExists === true){
          let uuid = this.UUID;
          let isFirst = this.getPreferences(pieceType, uuid + key)
          if (typeof (isFirst) === "undefined"){
            this.setPreferences(pieceType, uuid + key, this.VALUE_TYPE.BOOL, true);
          }
        }
        this.removePreferences(pieceType, key);
      }
      else {
        this.setPreferences(pieceType, key, type, data);
      }
    }
  }

  /**
   * 認証情報を保存する(情報は同じPiece間で共有される)
   * 本APIを使用する場合はPieceJsonのserviceProxyのauthTypeに'other'を必ず設定すること
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @param {JSON} data 保存対象のAuth情報
   * @param {callback} callback コールバック関数
   */
  setAuthInfo(pieceType, data, callback) {
    this.log("setAuthInfo", `in: type=${pieceType}, data=${data}`);

    let authInfo;
    switch (pieceType) {
      case this.PIECE_TYPE.TRIGGER:
        this.triggerAuthInfo = data;
        authInfo = this.triggerAuthInfo;
        break;
      case this.PIECE_TYPE.SERVICE:
        this.serviceAuthInfo = data;
        authInfo = this.serviceAuthInfo;
        break;
      case this.PIECE_TYPE.ACTION:
        this.actionAuthInfo = data;
        authInfo = this.actionAuthInfo;
        break;
      default:
        this.log("setAuthInfo", `Piece Name Error`);
        return;
    }

    this.setAuthInfoCallbackList[this.setAuthInfoCallbackIds] = callback;

    this._nativeCall({
      api: "setAuthInfo",
      callBackId: this.setAuthInfoCallbackIds,
      values: {
        "authInfo": authInfo,
        "pieceType": pieceType
      }
    });
    this.setAuthInfoCallbackIds++;
  }

  /**
   * 認証情報を取得する(情報は同じPiece間で共有される)
   * 本APIを使用する場合はPieceJsonのserviceProxyのauthTypeに'other'または'oauth2'を必ず設定すること
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @return {JSON} 認証情報
   */
  getAuthInfo(pieceType) {
    this.log("getAuthInfo", `in: type=${pieceType}`);
    switch (pieceType) {
      case this.PIECE_TYPE.TRIGGER:
        return this.triggerAuthInfo;
      case this.PIECE_TYPE.SERVICE:
        return this.serviceAuthInfo;
      case this.PIECE_TYPE.ACTION:
        return this.actionAuthInfo;
      default:
        this.log("getAuthInfo", `Piece Name Error`);
        return;
    }
  }

  /**
   * iiideaのicon情報を取得する
   * @param {string} type 対象となるPiece種別(ICON_TYPE)の値を指定
   * @param {callback} callback コールバック関数
   */
  getIconImage(type, callback) {
    this.log("getIconImage", `in: type=${type}`);
    this.getIconImageCallbackList[this.getIconImageCallbackIds] = callback;

    this._nativeCall({
      api: "getIconImage",
      callBackId: this.getIconImageCallbackIds,
      values: {
        "type": type
      }
    });
    this.getIconImageCallbackIds++;
  }

  /**
   * 指定したPreferencesプロパティ情報が設定任意か否かを取得する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @param {string} key 保存情報のフィールド名
   * @return {boolean} プロパティが設定任意の場合はtrue(falseの場合は設定必須のプロパティ)
   */
  isOptional(pieceType, key) {
    this.log("isOptionalProperty", `in: pieceType=${pieceType}, key=${key}`);
    let ret = false;
    switch (pieceType) {
      case this.PIECE_TYPE.TRIGGER:
        if (typeof (this.triggerOptPref) !== "undefined") {
          ret = this.triggerOptPref[key];
        }
        break;
      case this.PIECE_TYPE.SERVICE:
        if (typeof (this.serviceOptPref) !== "undefined") {
          ret = this.serviceOptPref[key];
        }
        break;
      case this.PIECE_TYPE.ACTION:
        if (typeof (this.actionOptPref) !== "undefined") {
          ret = this.actionOptPref[key];
        }
        break;
      default:
        this.log("isOptionalProperty", `Piece Name Error`);
        return ret;
    }
    return ret;
  }

  /**
   * 指定したPreferencesプロパティ情報にデフォルト設定が設定されているか否かを取得する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   * @param {string} key 保存情報のフィールド名
   * @return {boolean} デフォルト設定がある場合はtrue
   */
   defaultExists(pieceType, key) {
    this.log("defaultExists", `in: pieceType=${pieceType}, key=${key}`);
    let ret = false;
    switch (pieceType) {
      case this.PIECE_TYPE.TRIGGER:
        if (typeof (this.triggerDefaultExists) !== "undefined") {
          ret = this.triggerDefaultExists[key];
        }
        break;
      case this.PIECE_TYPE.SERVICE:
        if (typeof (this.serviceDefaultExists) !== "undefined") {
          ret = this.serviceDefaultExists[key];
        }
        break;
      case this.PIECE_TYPE.ACTION:
        if (typeof (this.actionDefaultExists) !== "undefined") {
          ret = this.actionDefaultExists[key];
        }
        break;
      default:
        this.log("defaultExists", `Piece Name Error`);
        return ret;
    }
    return ret;
  }

  /**
   * 認証用ページを表示する
   * 本APIは今後クローズする予定なので使用しないこと。
   * @param {string} url 認証ページのURL
   * @param {string} redirectUrlScheme リダイレクトURLのスキーム
   * @param {showAuthPageCallback} callback コールバック関数
   */
  showAuthPage(url, redirectUrlScheme, callback) {
    this.log("showAuthPage", `in: url=${url}, redirectUrlScheme=${redirectUrlScheme}`);
    this.showAuthPageCallbackList[this.showAuthPageCallbackIds] = callback;
    this._nativeCall({
      api: "showAuthPage",
      callBackId: this.showAuthPageCallbackIds,
      values: {
        "url": url,
        "redirectUrlScheme": redirectUrlScheme
      }
    });
    this.showAuthPageCallbackIds++;
  }

  /**
   * パーミッションの許可要求をネイティブへ通知する
   * @param {string} permissionType permissionType
   */
  permissionRequestNotification(permissionType) {
    this.log("permissionRequestNotification", `in: permissionType=${permissionType}`);
    this._nativeCall({
      api: "permissionRequestNotification",
      values: {
        "permissionType": permissionType
      }
    });
  }

  /**
   * パーミッション(Auth)の許可要求をネイティブへ通知する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   */
  permissionAuthRequestNotification(pieceType) {
    this.log("permissionAuthRequestNotification", `in: pieceType=${pieceType}`);
    this._nativeCall({
      api: "permissionAuthRequestNotification",
      values: {
        "pieceType": pieceType
      }
    });
  }

  /**
   * パーミッション(Auth)の削除要求をネイティブへ通知する
   * @param {string} pieceType 対象となるPiece種別(PIECE_TYPE)の値を指定
   */
  deletePermissionAuthRequestNotification(pieceType) {
    this.log("deletePermissionAuthRequestNotification", `in: pieceType=${pieceType}`);
    this._nativeCall({
      api: "deletePermissionAuthRequestNotification",
      values: {
        "pieceType": pieceType
      }
    });
  }

  /**
   * iiideaの削除要求をネイティブへ通知する
   */
  deleteIiideaRequestNotification(callback) {
    this.log("deleteIiideaRequestNotification",`in`);
    this.deleteIiideaCallbackList[this.deleteIiideaCallbackIds] = callback;
    this._nativeCall({
      api: "deleteIiideaRequestNotification",
      callBackId: this.deleteIiideaCallbackIds
    });
    this.deleteIiideaCallbackIds++;
  }

  /**
   * iiideaのactivate/diactivate要求をネイティブへ通知する
   * changeActivateAppletStatusを実行する時、現在のiiideaの状態がactivateであればdiactivateされ、diactivateであればactivateされる。
   */
  changeActivateAppletStatus(callback) {
    this.log("changeActivateAppletStatus", `in`);
    this.changeActivateStatusCallbackList[this.changeActivateStatusCallbackIds] = callback;
    this._nativeCall({
      api: "changeActivateAppletStatus",
      callBackId: this.changeActivateStatusCallbackIds
    });
    this.changeActivateStatusCallbackIds++;
  }

  /**
   * Preferenceをcloseする要求をネイティブへ通知する
   */
  closePreferencesRequestNotification() {
    this.log("closePreferencesRequestNotification",`in`);
    this._nativeCall({
      api: "closePreferencesRequestNotification"
    });
  }
  
  /**
   * HTTP通信を要求する
   * @param {string} method リダイレクトURL
   * @param {string} url 通信先URL
   * @param {JSON} headers ヘッダ情報
   * @param {JSON} parameters パラメータ情報
   * @param {requestCommunicationCallback} callback コールバック関数
   */
  requestCommunication(method, url, headers, parameters, callback) {
    this.log("requestCommunication", `in: method=${method}, url=${url}, headers=${headers}, parameters=${parameters}`);
    this.requestCommCallbackList[this.requestCommCallbackIds] = callback;
    this._nativeCall({
      api: "requestCommunication",
      callBackId: this.requestCommCallbackIds,
      values: {
        "method": method,
        "url": url,
        "headers": headers,
        "parameters": parameters
      }
    });
    this.requestCommCallbackIds++;
  }

  /**
   * ログを出力する
   * @param {string} func 関数名
   * @param {string} message 出力メッセージ
   */
  log(func, message) {
    let log = '[' + func + '] ' + message;
    console.log(log);
    this._nativeCall({
      api: "log",
      values: {
        "message": log
      }
    });
  }

  /**
   * GoogleMapからのコールバック
   */
  _googleMapCallback() {
    this.log("_googleMapCallback", `in`);
    this.isMapInitialized = true;
    for (let key in this.googleMapCallbackList) {
      this.googleMapCallbackList[key]();
      delete this.googleMapCallbackList[key];
    }
  }

  /**
   * ネイティブ通信用関数(外部からコール禁止)
   */
  _nativeCall(value) {
    value.timestamp = new Date().getTime()
    if (typeof webkit.postMessage == 'function') {
      // Android
      webkit.postMessage(JSON.stringify(value));
    }
    else if (typeof webkit.messageHandlers.ERSDK.postMessage== 'function'){
      // iOS
      webkit.messageHandlers.ERSDK.postMessage(value);
    }
  }

  /**
   * ネイティブ通信用関数(外部からコール禁止)
   */
  _nativeCallback(value) {
    this.log("_nativeCallback", `in: value=${value}`);
    let jsonData = JSON.parse(value);
    switch (jsonData.api) {
      case 'nativeInit':
        this.log("_nativeCallback", `nativeInit`);
        this.isInitialized = true;
        for (let key in this.initCallbackList) {
          this.initCallbackList[key]();
          delete this.initCallbackList[key];
        }
        break;
      case 'setAuthInfo':
        this.log("_nativeCallback", `setAuthInfo`);
        if (jsonData.callBackId in this.setAuthInfoCallbackList) {
          this.setAuthInfoCallbackList[jsonData.callBackId](jsonData.values);
          delete this.setAuthInfoCallbackList[jsonData.callBackId];
        }
        break;
      case 'getIconImage':
        this.log("_nativeCallback", `getIconImage`);
        if (jsonData.callBackId in this.getIconImageCallbackList) {
          this.getIconImageCallbackList[jsonData.callBackId](jsonData.values);
          delete this.getIconImageCallbackList[jsonData.callBackId];
        }
        break;
      case 'requestCommunication':
        this.log("_nativeCallback", `requestCommunication`);
        if (jsonData.callBackId in this.requestCommCallbackList) {
          this.requestCommCallbackList[jsonData.callBackId](jsonData.values);
          delete this.requestCommCallbackList[jsonData.callBackId];
        }
        break;
      case 'showAuthPage':
        this.log("_nativeCallback", `showAuthPage`);
        if (jsonData.callBackId in this.showAuthPageCallbackList) {
          this.showAuthPageCallbackList[jsonData.callBackId](jsonData.values.redirectUrl);
          delete this.showAuthPageCallbackList[jsonData.callBackId];
        }
        break;
      case 'deleteIiideaRequestNotification':
        this.log("_nativeCallback", `deleteIiideaRequestNotification`);
        if (jsonData.callBackId in this.deleteIiideaCallbackList) {
          this.deleteIiideaCallbackList[jsonData.callBackId](jsonData.values);
          delete this.deleteIiideaCallbackList[jsonData.callBackId];
        }
        break;
      case 'changeActivateAppletStatus':
        this.log("_nativeCallback", `changeActivateAppletStatus`);
        if (jsonData.callBackId in this.changeActivateStatusCallbackList) {
          this.changeActivateStatusCallbackList[jsonData.callBackId](jsonData.values);
          delete this.changeActivateStatusCallbackList[jsonData.callBackId];
        }
        break;  
    }
  }
}
let erSDK = new ERSDK();

/**
 * erSDKの初期化完了を通知する
 *
 * @callback initCallback
 */

 /**
 * googleMapの初期化完了を通知する
 *
 * @callback googleMapCallback
 */

/**
 * カレンダーの日付が選択されたら通知する
 *
 * @callback updateCalendarSelect
 * @param {number} year 選択された日付の年(yyyy)
 * @param {number} month 選択された日付の月(mm)
 * @param {number} day 選択された日付の日(dd)
 */

/**
 * 認証用ページからのリダイレクト要求を通知する
 *
 * @callback showAuthPageCallback
 * @param {string} redirectUrl リダイレクトURL
 */

/**
 * HTTP通信の結果を通知する
 *
 * @callback requestCommunicationCallback
 * @param {JSON} values HTTP通信レスポンス情報
 */
